% function [V,l,M]=DLB(level,creal, cimag, meas, plotmth, toplot, writeme) 
% ------------------------------------------
% level is the level of the approximation 
% meas is either 'e' for equilibrium or 'c' for conformal 
% plotmth is a character 
%      -'r'- plots just the ray information (2 to a page) 
%      -'s'- plots ray and support information (4 to a page) (default)
%      -'c'- plots ray, support and inner circle information (4 to a page) 
%      -'h'- plots the pretty eigenfunctions 
%      -'b'- makes the color fun 3-D plots (not writen out)
%      -'n'- none turns off ploting
%      if longer string specified only first character will be used  
% toplot is a vector specifying the functions to be plotted 
% writeme (1, or 0) tells the function wether to write out the eigenvalues (if
%     toplot is specified default is 0 else default is 1) 

% Example: DLB(6,'c', 'c', [1:10,39:40], 1 ); will compute the spectrum for
% the level 6 approximation to the laplacian with conformal measure and
% plot the ray parameterization, support and restriction to the inner circle for
% eigenfunctions 1-10 and 39 and 40, it will the save the all the
% eigenvalues that the plots of the specified functions. 

% Uses LapBW.m to set up points, exterior rays and weights then uses this
% information to form a discrete estimate for the laplacian (using the
% average of even and odd energy's) and evaluate it's eigenvalues. 

function [V,l,M]=DLB(level, creal, cimag, meas, plotmth, toplot, writeme) 
% We compute the bilinear forms for energy and measure for our graph
% approximation of the Basilica(the Julia set for the polynomial z^2-1).
% We then create the matrix approximation to the Laplacian for the specified measure and
% evalutate it's spectra.

% For our approximation we consider the mapping from the Basilica on to the
% circle defined by the exterior rays, because of the topology of the
% bascilica we must make certian identifications (matches) between points
% circle.(see LapBW for more details). For the purposes of computation we
% consider Energy and Measure as acting on vectors composed of the free
% and then map the resulting eigenvectors back onto the bascilica by making
% the necessary identifications.  

% Looking at the pullback of the standard energy on the circle through this
% exterior angle map we see that there are two natural energies on the
% Bascilica, one corresponding to the pullback on even level approximations
% and one, that on odd levels. We take our energy to be the average of
% these energies. We create the energy numerically by creating the matrix
% for the product of difference quotients on an even level and then on an
% odd level, modifying the smaller matrix so that it opperates on the
% larger set and then averaging them. 

% There are two natural choices for the measure, equilibrium measure, the
% pull back of lebesque measure, or conformal measure which incorporates
% information on the dynamics of the set (see LapBW for more details). By
% specifying 'e' or 'c' we can compute the Laplacian corresponding to
% either measure. We create the discrete approximation to the measure using
% weights. 

% uses LapBW.m 

% Taryn Flock Summer 2008

%% Input Processing %% 
% -----------------------------------------------------------------
% assign default values depending on number of inputs specified
 if nargin<1 
    fprintf('Running an Example Case: Level 7 c=-1 equilibrium measure\n')
    level=7;
    creal=-1;
    cimag=0;
    meas='e';
    plotmth='c';
    toplot=[1:4,47:51];
    writeme=0;    
elseif nargin==2
    fprintf('Running with equilibrium measure\r')
    cimag=0;
    meas='e';
    plotmth='s';
    toplot=1:10; 
    writeme=0; 
elseif nargin==3;
    fprintf('Running with equilibrium measure\r')
    meas='e';
    plotmth='s';
    toplot=1:10;
    writeme=0;
elseif nargin==4;
    plotmth='s';
    toplot=1:10;
    writeme=0;
 elseif nargin==5;
     toplot=1:10;
     writeme=0;
 elseif nargin==6
     writeme=0;
 end
 

 r=sqrt(2);  % renormalization factor for the energy 

%%  Calculate Previous Energy Matrix    %%
% ---------------------------------------------------------------------
% Here we create the matrix corresponding to the difference quotient on 
% the level prior

% Get set up info 
[rays1,vals1,weights1,matches1]=LapBW(level-1,creal,cimag,'n');
numpoints1=length(rays1); 

% Calculate the differences btw ray angles- the denominators
Drays1=[rays1,0]-[rays1(numpoints1)-(3*2^(level-1)),rays1];
Drays1=Drays1(1:numpoints1);

% Calculate Old Energy Matrix -- the 1st difference quotient with theta
% factor
OE1=diag(-1./Drays1,0)+diag(1./Drays1(2:numpoints1),1);
OE1(numpoints1,1)=1/Drays1(1);

OE1=6*OE1;  % To normalize our matrix 

% the 2nd difference quotient
OE2=diag(-1*ones(1,numpoints1),0)+diag(ones(1,numpoints1-1),-1);
OE2(1,numpoints1)=1;

OE=r^(level-1)*OE1*OE2; % take product and normalize 

%% Now Examine the Level of Interest  %%
% -----------------------------------------------------------
% repeat the process above but on the new level, also modify the previous
% energy matrix to opperate at this higher level

% Get info for this level 
[rays,vals,weights,matches]=LapBW(level,creal,cimag,'n');
numpoints=length(rays); 

% Assign the appropriate weights for the specified measure
if (meas=='e') 
    weights=ones(1,numpoints )/numpoints;
elseif (meas~='c') 
    error('Please specify a valid measure, either c, conformal, or e, equalibrium')
end 

% Raise the energy matrix from the previous level to opperate at the new
% higher level
Eupped1=zeros(numpoints,numpoints/2);
Eupped1(~mod(rays,2),:)= OE; 
Eupped=zeros(numpoints,numpoints);
Eupped(:,~mod(rays,2))=Eupped1; 

% Make Matches Map (converts from vectors of free variables to
% eigenfunctions on the bascilica by making the necessary identifications).
M =zeros(numpoints,numpoints/2);
for i=1:numpoints/2 
      M(matches(i,:),i)=[1,1];
end 

% Calculate the differences btw ray angles
Drays=[rays,0]-[rays(numpoints)-(3*2^level),rays];
Drays=Drays(1:numpoints);

% Calculate Energy Matrix
E1=diag(-1./Drays,0)+diag(1./Drays(2:numpoints),1);
E1(numpoints,1)=1/Drays(1);
E1=6*E1;  % to get the normalization correct

E2=diag(-1*ones(1,numpoints),0)+diag(ones(1,numpoints-1),-1);
E2(1,numpoints)=1;

E=(1/2)*M'*(r^level*E1*E2+Eupped)*M;   %Use average of Eupped and E.  

% Calculate 'Integral' wrt measure matrix
G=M'*diag(weights)*M;

% Laplacian Matrix
L=inv(G)*E;

% Calculate and order eigenvalues
[V,D]=eig(L);
[l,I]=sort(nonzeros(D));
V=V(:,I);
V=CleanVB(V,M,level,creal,cimag,meas);


%% Result Processing 
% ------------------------------------------------
% Write out eigenvalue data 
if writeme==1; 
    filename=strcat('B',meas,'l',num2str(level),'.out');
    fid=fopen(filename,'w');
    if (fid==-1); error('Bad Filename'); end
    fprintf(fid,'% f\n', l);
    fclose(fid);
end 

% Make Pretty Pictures of the eigenfunctions
MaxPics=length(toplot); 
if plotmth=='c' 
    [incirc,incirc2]=InCircle(level,creal,cimag);
    clear incirc2
    %incirc=incirc(logical(mod(1:length(incirc),2)));
    incirc=logical(incirc);
    for j=1:ceil(MaxPics/4); 
        figure;
        for i=1:min(4,MaxPics-4*(j-1))
            eignum = toplot(i+4*(j-1));
            subplot(4,3,3*i-2);
            % Plot Rays 
            eigfun=M*V(:,eignum);
            plot(rays/(max(rays)+1),eigfun, 'LineWidth',1);
            title(strcat('Num',num2str(eignum)));
            axis square
            axis tight
            
            subplot(4,3,3*i-1);
            % Look at sign of sufficiently nonzero points 
            goodpt=abs(eigfun)>(max(abs(V(:,1)))-min(abs(V(:,1))));
            %goodpt=(abs(eigfun)>(max(abs(eigfun))/100));
            goodpt=goodpt&(abs(eigfun)>(max(abs(eigfun))/1000));
            signpt=sign(eigfun(goodpt));
            scatter(real(vals(goodpt)),imag(vals(goodpt)),5*abs(signpt),signpt, 'filled')
            title(strcat('Eigval',num2str(l(eignum))))
%             cm = [.2 .2 .2; .5 .5 .5];  % Use to make the graphs black
%             and white 
%             colormap(cm);
             colormap('winter');
            axis([-2 2 -1.5 1.5 ]);
            axis square 
                     
            subplot(4,3,3*i);
            % Plot Inner Circle 
            eigfunc=eigfun(incirc);
            eigfunc=eigfunc(logical(mod(1:length(eigfunc),2)+[zeros(1,length(eigfunc)-1),1]));% thin out matches 
            plot((1:length(eigfunc))/(length(eigfunc)+1),eigfunc, 'LineWidth',1);
            axis square
            axis tight
            if sum(goodpt(incirc))==0
                ylim([-abs(max(V(:,2))),abs(max(V(:,2)))])
            end
        end
        set(gcf,'PaperPositionMode','auto');
        set(gcf,'Position',[1,1,560,700]);
        if writeme==1 
            filename = strcat('B',num2str(level),meas,'Pic',num2str(j),'.png'); 
            print('-dpng', filename)
            close
        end 
    end
elseif plotmth=='s' 
    for j=1:ceil(MaxPics/4)
        figure;
        for i=1:min(4,MaxPics-4*(j-1))
            eignum = toplot(i+4*(j-1));
            subplot(4,2,2*i-1);
            % Plot Rays 
            eigfun=M*V(:,eignum);
            plot(rays,eigfun,'k','LineWidth',1);
            title(strcat('Num',num2str(eignum)))
            axis square
            axis tight
            
            subplot(4,2,2*i);
            % Look at sign of sufficiently nonzero points 
            goodpt=abs(eigfun)>2*(max(abs(V(:,1)))-min(abs(V(:,1))));
            goodpt=goodpt&(abs(eigfun)>(max(abs(eigfun))/100));
            signpt=sign(eigfun(goodpt));
            scatter(real(vals(goodpt)),imag(vals(goodpt)),5*abs(signpt),signpt, 'filled')
            title(strcat('Eigval',num2str(l(eignum))))
            cm = [.2 .2 .2; .5 .5 .5];
            colormap(cm);
            axis([-2 2 -1.5 1.5 ]);
            axis square     
        end
        set(gcf,'PaperPositionMode','auto');
        set(gcf,'Position',[1,1,560,700]);
    if writeme==1 
        filename = strcat('Q',num2str(creal),'L',num2str(level),meas,'Pic',num2str(j),'.png'); 
        print('-dpng', filename)
        close
    end
    end
elseif plotmth=='r' 
      for j=1:ceil(MaxPics/3)
        figure;
        for i=1:min(3,MaxPics-3*(j-1))
            eignum = toplot(i+3*(j-1));
            subplot(3,1,i);
            % Plot Rays 
            eigfun=M*V(:,eignum);
            plot(rays/(max(rays)+1),eigfun,'k','LineWidth',1);
            title(strcat('Num ',num2str(eignum),' EigVal ',num2str(l(eignum))))
            axis tight
        end
        set(gcf,'PaperPositionMode','auto');
        set(gcf,'Position',[1,1,560,700]);
        if writeme==1 
            filename = strcat('B',num2str(level),meas,'Pic',num2str(j),'.png'); 
            print('-dpng', filename)
            close
        end
        if toplot(i)==47 
            subplot(3,1,2);
            goodpt=(abs(eigfun)>(max(abs(eigfun))/100));
            plotter=eigfun.*goodpt;
            plot(rays/(max(rays)+1),plotter,'k', 'LineWidth',1);
            title(strcat('Num',num2str(eignum)));
            axis tight
        end
        
      end
elseif plotmth=='h'      
        for i=toplot;
            figure;
            scatter3(real(vals),imag(vals),M*V(:,i),10,6*M*V(:,i),'filled');
        end
elseif plotmth=='b'
  for i=toplot
      figure;
      scatter3(real(vals),imag(vals),M*V(:,i),15,M*V(:,i),'filled');
      axis([-2 2 -1.5 1.5 min(V(:,i)), max(V(:,i))]) 
  end
elseif plotmth~='n' 
    fprintf('Invalid plot method please specify r,c,s or n\r')
end

