% function [outrays,outvals,outwghts,matches]=LapBW(level,creal,cimag,plotmth)
% -------------------------------------------------------
%
% Does the set up work for computing the laplacian on the Bascilica Julia
% Set, uses a descrete approximation to the conformal measure as the weights 
%
% level speficies the number of times to apply the polynomial
% plotmth(2) is a character
%      -'h'-height: plots measure in 3D
%      -'b'-bubble: plots measure in 2D
%      -'d'- deriv: plots 2|z|
%      -'c'- cummulative sum of nu against ray angle 
%      -'u'- nu: plot nu against ray angle 
%      -'r'-raycheck:plots exterior rays as height
%      -'a'- all:  spits out all plots 
%      -'n'- none(default)turns off plot, useful as often a helper func.
%   if you would like to specify two of the three plot methods feel free
%   to do so 
%
% Example [r,v,w]=LapBW2(5,-1,0,'br']); 

function [outrays,outvals,outwghts,outmatches]=LapBW(level,creal,cimag,plotmth) 
% This function generates the a graph approximation of bascilica julia set
% (c=-1). The information returned is the exterior rays, complex plane 
% locations, and discrete weights for the points. 

% We generate our graph approximation by taking pre-images of the fixed
% point P=z^2+c that occurs on the julia set. Level 0 is precisely that
% point. Level m consists of all the preimages of points at level m-1. 

% Weights are assigned to be the discrete approximation to the conformal
% measure. W(z)=W(P(z))*|z|^alpha.  Where alpha is the haussdorf measure
% of the julia set (assuming that the julia has zero area). 
% ----------------------------------------------------------------------
if nargin<1
    error('I need more info! please specify level')
elseif nargin==1 % if no c value specifed use -1
    creal=-1;
    cimag=0;
    plotmth='n';
elseif nargin==2 % if only real part specified set cimag=0
    cimag=0;
    plotmth='n';
elseif nargin==3 % if no plot method specified don't plot
    plotmth='n';
end

c=creal+sqrt(-1)*cimag;
if (c==-1) 
    alpha=1.2683; % Estimate of Haussdorf measure
elseif (c==-1+.15i)
    alpha=1.3052;
else 
    alpha=1;
    fprintf('Haussdorf dimmension for given c not known, conformal measure data invalid\n')
end

numpoints = 2^(level+1); % number of points at a level

rays    = zeros(2, numpoints); % External ray values
values  = zeros(2, numpoints); % Point values
weights = zeros(2, numpoints); % Given by the conformal measure
imgindx  = zeros(2, numpoints); % Records the index of the image of a point

% matches  = zeros(numpoints/2,2);  % records the match values for the final rays matrix 

center =  zeros(1,level-1);
% Set values for the 0th iteration
if (level>=0)
    rays(1,1:2)   = [1 2];  % really just numerator, denominator is 3
    values(1,1)   = min(roots([1 -1 c]));
    values(1,2)   = values(1,1); 
    weights(1,1:2)= [1,1];  % yep, this is arbitrary (but they must be equal)
%     matches(1,:)= [1,2];  % these points match
else
    error('Sorry cannot go backward-anywise it would be rather boring, the images of a fixed point....')
end 

% 1st iteration, it's unique one of the preimages is a duplicate
if ( level>=1 )
    values(2,:) = values(1,:);
    if (c==-1)
        values(2,3:4)= max(roots([1 0 c-values(1,1)]))*ones(1,2);
    else
        values(2,3:4)= min(roots([1 0 c-values(1,1)]))*ones(1,2);
    end
    
    % Probably not the most efficient way to sort here, but miny version of
    % general case
    rays(2,:)  = 2*rays(1,:);
    rays(2,3)  = 1; % 'half' of each old angle
    rays(2,4)  = 5; % 'half' of each old angle + pi 
    
    imgindx(2,1:4) = ones(1,4);  % at this level every point maps to the first
    weights(2,1:4) = weights(1,imgindx(2,1:4)).*abs(2*values(2,1:4)).^(-alpha);
   
    % Update Arrays
    values(1,:) = values (2,:);
    weights(1,:)= weights(2,:);
    imgindx(1,:) = imgindx (2,:);
    rays(1,:)   = rays   (2,:); 
    center(1) = values(1,4);
%     matches(2,:) = [3,4];
    
elseif (level==0)
    fprintf('Really, you dont want me to iterate?\r')
else 
    error('Out of curiousity, what would a fractional interation be?')
end

% More iterations 
% Some facts that make the code make more sense
%     + iteration i there are 2^(i+1) points 
%     + at each level the donominator value is 3*2^i

for i=2:level    
% Preserve old values
 %   values(2,:) = values(1,:); Redundant lines
 %   imgindx(2,:) = imgindx(1,:);
    rays(2,:) = 2*rays(1,:);
 	
% Calculate New Values   
    newval = [sqrt(values(1,2^(i-1)+1:2^i)-c),...
                -sqrt(values(1,2^(i-1)+1:2^i)-c)]; 
            
    checkpts = real(newval)>real(center(i-1));
    [maxx,inx] =  max(imag(newval(checkpts)));
    
    center(i) = newval(inx);
    centerline = [-center(i:-1:1),center(1:i)];
    
    [testers,ibb]=connectdots(centerline,newval);
    helper=imag(newval)>testers;  % get points above the line 
    helper(ibb)=~helper(ibb);
    
    px = (newval==center(i)).*(mod(1:length(newval),2)==1);
    % for the center line with negative x assign two small angle values 
    nx = (newval==-center(i)).*(mod(1:length(newval),2)~=1); 
    helper= helper+px+nx; 
    
      
    newray = (helper).*[rays(1,2*2^(i-2)+1:2*2^(i-1)),rays(1,2*2^(i-2)+1:2*2^(i-1))];
    newray = newray + (~helper).*[rays(1,2*2^(i-2)+1:2*2^(i-1))+3*2^(i-1),...
                                          rays(1,2*2^(i-2)+1:2*2^(i-1))+3*2^(i-1)];
    
%     matches(2^(i-1):2^i,:) = [(2^i:2^(i+1)).*(mod(1:2^i,2)==1),(2^i:2^(i+1)).*(mod(1:2^i,2)==1)+1];         

 
     
% Put it together    
    values(2,2^i+1:2^(i+1)) = newval;
    rays(2,2^i+1:2^(i+1))   = newray;
    imgindx(2,2^i+1:2^(i+1)) = [2^(i-1)+1:2^i,2^(i-1)+1:2^i];
    weights(2,1:2^(i+1)) = weights(1,imgindx(2,1:2^(i+1))).*((2*abs(values(2,1:2^(i+1)))).^(-alpha));

    % Update Arrays
    values(1,:) = values (2,:);
    weights(1,:)= weights(2,:);
    imgindx(1,:) = imgindx (2,:);
    rays(1,:)   = rays   (2,:); 
end

% Sort by ray order, tidy up and find matches
outrays    = rays(2,:);
[outrays, I] = sort(outrays); 
outvals  = values(2,I);
outwghts = weights(2,I)/sum(weights(2,:));

% Calculate Matches (Brute force...) 
unused=1:numpoints; %store unused indices
matches=zeros(numpoints/2,2);

for i=1:numpoints/2;  % evaluate matches for half the points
    % calculate match at first unused index
    matchindx    = find(outvals==outvals(min(unused)));
    matches(i,:) = matchindx;
    unused(matchindx)=[numpoints+1,numpoints+1];
end    
outmatches=matches;
% All the Plotting stuff 
% ------------------------------------------------------------------------
if (plotmth=='n')
    return;
end

for i=1:length(plotmth)
    if (plotmth(i)=='h'||plotmth(i)=='a') 
        % Height plot
        % uses the measure as the height
        scatter3(real(outvals),imag(outvals),outwghts,4,outwghts,'*')
        prettyify;
        colorbar;
        title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='b'||plotmth(i)=='a') 
        % Bubble Plot
        % use measures as the size of the weight
        figure;
        scatter(real(outvals),imag(outvals),3*numpoints*outwghts,outwghts,'filled');
        prettyify
        axis equal
        %title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='d'||plotmth(i)=='a') 
        % Derivative Plot
        % use 2|z| as the size of the weight
        figure;
        scatter(real(outvals),imag(outvals),21*abs(outvals),abs(outvals),'filled');
        prettyify
        axis equal
        % title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='u'||plotmth(i)=='a') 
        % Nu Plot
        % plot ray and measure as height
        figure;
        plot(outrays/max(outrays), outwghts,'k');
        % title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='c'||plotmth(i)=='a') 
        % CumSum Plot
        % plot ray and measure as height
        figure;
        plot(outrays/max(outrays),cumsum(outwghts),'k');
        ylim([0,1])
        % title(strcat('Alpha',num2str(alpha),'Iteration',num2str(level)));
    end
    
    if(plotmth(i)=='r'||plotmth(i)=='a') 
        % Check rays
        % plot rays as height, if rays are computed incorrectly 
        % you can expect to see points connected to the opposite side of
        % the quasicircle--can happen on asymmetric quasicircles
        figure;
        plot3(real(outvals),imag(outvals), outrays)
        title('Exterior ray angle check')
    end
    
end